import PropTypes from 'prop-types';
import React from 'react';
import { Popper } from 'react-popper';
import { noop } from '../utils';
export var Align;
(function (Align) {
    Align["JUSTIFY"] = "justify";
    Align["LEFT"] = "left";
    Align["RIGHT"] = "right";
})(Align || (Align = {}));
const SafeElement = typeof Element === 'undefined' ? noop : Element;
const propTypes = {
    align: PropTypes.oneOf(Object.values(Align)),
    children: PropTypes.func.isRequired,
    dropup: PropTypes.bool,
    flip: PropTypes.bool,
    isMenuShown: PropTypes.bool,
    positionFixed: PropTypes.bool,
    referenceElement: PropTypes.instanceOf(SafeElement),
};
const defaultProps = {
    align: Align.JUSTIFY,
    dropup: false,
    flip: false,
    isMenuShown: false,
    positionFixed: false,
};
function getModifiers(props) {
    return {
        computeStyles: {
            enabled: true,
            fn: (data) => {
                return {
                    ...data,
                    styles: {
                        ...data.styles,
                        width: props.align !== Align.RIGHT && props.align !== Align.LEFT
                            ?
                                `${data.offsets.reference.width}px`
                            : data.styles.width,
                    },
                };
            },
        },
        flip: {
            enabled: props.flip,
        },
        preventOverflow: {
            escapeWithReference: true,
        },
    };
}
export function getPlacement(props) {
    const x = props.align === Align.RIGHT ? 'end' : 'start';
    const y = props.dropup ? 'top' : 'bottom';
    return `${y}-${x}`;
}
const Overlay = ({ referenceElement, ...props }) => {
    if (!props.isMenuShown) {
        return null;
    }
    return (React.createElement(Popper, { modifiers: getModifiers(props), placement: getPlacement(props), positionFixed: props.positionFixed, referenceElement: referenceElement }, ({ arrowProps, ref, outOfBoundaries, ...popperProps }) => props.children({
        ...popperProps,
        innerRef: ref,
        inputHeight: referenceElement ? referenceElement.offsetHeight : 0,
    })));
};
Overlay.propTypes = propTypes;
Overlay.defaultProps = defaultProps;
export default Overlay;
